"""Feishu/Lark channel implementation using lark-oapi SDK with WebSocket long connection.

使用 lark-oapi SDK 实现飞书 Channel，通过 WebSocket 长连接接收消息。
无需公网 IP 或 webhook 配置。
"""

import asyncio
import json
import logging
import os
import re
import threading
from collections import OrderedDict
from pathlib import Path
from typing import Any, Optional

from iflow_bot.bus.events import OutboundMessage
from iflow_bot.bus.queue import MessageBus
from iflow_bot.channels.base import BaseChannel
from iflow_bot.channels.manager import register_channel
from iflow_bot.config.schema import FeishuConfig

try:
    import lark_oapi as lark
    from lark_oapi.api.im.v1 import (
        CreateFileRequest,
        CreateFileRequestBody,
        CreateImageRequest,
        CreateImageRequestBody,
        DeleteMessageReactionRequest,
        CreateMessageRequest,
        CreateMessageRequestBody,
        CreateMessageReactionRequest,
        CreateMessageReactionRequestBody,
        Emoji,
        GetMessageResourceRequest,
        PatchMessageRequest,
        PatchMessageRequestBody,
        P2ImMessageReceiveV1,
    )
    FEISHU_AVAILABLE = True
except ImportError:
    FEISHU_AVAILABLE = False
    lark = None  # type: ignore
    Emoji = None  # type: ignore


logger = logging.getLogger(__name__)


# Message type display mapping
MSG_TYPE_MAP = {
    "image": "[image]",
    "audio": "[audio]",
    "file": "[file]",
    "sticker": "[sticker]",
}


def _extract_share_card_content(content_json: dict, msg_type: str) -> str:
    """Extract text representation from share cards and interactive messages."""
    parts = []

    if msg_type == "share_chat":
        parts.append(f"[shared chat: {content_json.get('chat_id', '')}]")
    elif msg_type == "share_user":
        parts.append(f"[shared user: {content_json.get('user_id', '')}]")
    elif msg_type == "interactive":
        parts.extend(_extract_interactive_content(content_json))
    elif msg_type == "share_calendar_event":
        parts.append(f"[shared calendar event: {content_json.get('event_key', '')}]")
    elif msg_type == "system":
        parts.append("[system message]")
    elif msg_type == "merge_forward":
        parts.append("[merged forward messages]")

    return "\n".join(parts) if parts else f"[{msg_type}]"


def _extract_interactive_content(content: dict) -> list[str]:
    """Recursively extract text and links from interactive card content."""
    parts = []

    if isinstance(content, str):
        try:
            content = json.loads(content)
        except (json.JSONDecodeError, TypeError):
            return [content] if content.strip() else []

    if not isinstance(content, dict):
        return parts

    if "title" in content:
        title = content["title"]
        if isinstance(title, dict):
            title_content = title.get("content", "") or title.get("text", "")
            if title_content:
                parts.append(f"title: {title_content}")
        elif isinstance(title, str):
            parts.append(f"title: {title}")

    elements = content.get("elements")
    if isinstance(elements, list):
        for element in elements:
            parts.extend(_extract_element_content(element))

    card = content.get("card", {})
    if card:
        parts.extend(_extract_interactive_content(card))

    header = content.get("header", {})
    if header:
        header_title = header.get("title", {})
        if isinstance(header_title, dict):
            header_text = header_title.get("content", "") or header_title.get("text", "")
            if header_text:
                parts.append(f"title: {header_text}")

    return parts


def _extract_element_content(element: dict) -> list[str]:
    """Extract content from a single card element."""
    parts = []

    if not isinstance(element, dict):
        return parts

    tag = element.get("tag", "")

    if tag in ("markdown", "lark_md"):
        content = element.get("content", "")
        if content:
            parts.append(content)

    elif tag == "div":
        text = element.get("text", {})
        if isinstance(text, dict):
            text_content = text.get("content", "") or text.get("text", "")
            if text_content:
                parts.append(text_content)
        elif isinstance(text, str):
            parts.append(text)
        for field in element.get("fields", []):
            if isinstance(field, dict):
                field_text = field.get("text", {})
                if isinstance(field_text, dict):
                    c = field_text.get("content", "")
                    if c:
                        parts.append(c)

    elif tag == "a":
        href = element.get("href", "")
        text = element.get("text", "")
        if href:
            parts.append(f"link: {href}")
        if text:
            parts.append(text)

    elif tag == "button":
        text = element.get("text", {})
        if isinstance(text, dict):
            c = text.get("content", "")
            if c:
                parts.append(c)
        url = element.get("url", "") or element.get("multi_url", {}).get("url", "")
        if url:
            parts.append(f"link: {url}")

    elif tag == "img":
        alt = element.get("alt", {})
        parts.append(alt.get("content", "[image]") if isinstance(alt, dict) else "[image]")

    elif tag == "note":
        for ne in element.get("elements", []):
            parts.extend(_extract_element_content(ne))

    elif tag == "column_set":
        for col in element.get("columns", []):
            for ce in col.get("elements", []):
                parts.extend(_extract_element_content(ce))

    elif tag == "plain_text":
        content = element.get("content", "")
        if content:
            parts.append(content)

    else:
        for ne in element.get("elements", []):
            parts.extend(_extract_element_content(ne))

    return parts


def _extract_post_text(content_json: dict) -> str:
    """Extract plain text from Feishu post (rich text) message content.

    Supports two formats:
    1. Direct format: {"title": "...", "content": [...]}
    2. Localized format: {"zh_cn": {"title": "...", "content": [...]}}
    """
    def extract_from_lang(lang_content: dict) -> Optional[str]:
        if not isinstance(lang_content, dict):
            return None
        title = lang_content.get("title", "")
        content_blocks = lang_content.get("content", [])
        if not isinstance(content_blocks, list):
            return None
        text_parts = []
        if title:
            text_parts.append(title)
        for block in content_blocks:
            if not isinstance(block, list):
                continue
            for element in block:
                if isinstance(element, dict):
                    tag = element.get("tag")
                    if tag == "text":
                        text_parts.append(element.get("text", ""))
                    elif tag == "a":
                        text_parts.append(element.get("text", ""))
                    elif tag == "at":
                        text_parts.append(f"@{element.get('user_name', 'user')}")
        return " ".join(text_parts).strip() if text_parts else None

    # Try direct format first
    if "content" in content_json:
        result = extract_from_lang(content_json)
        if result:
            return result

    # Try localized format
    for lang_key in ("zh_cn", "en_us", "ja_jp"):
        lang_content = content_json.get(lang_key)
        result = extract_from_lang(lang_content)
        if result:
            return result

    return ""


@register_channel("feishu")
class FeishuChannel(BaseChannel):
    """Feishu/Lark channel using WebSocket long connection.

    Uses WebSocket to receive events - no public IP or webhook required.

    Requires:
    - App ID and App Secret from Feishu Open Platform
    - Bot capability enabled
    - Event subscription enabled (im.message.receive_v1)

    Attributes:
        name: Channel name identifier
        config: FeishuConfig configuration object
        bus: MessageBus instance
        _client: Lark client for API calls
        _ws_client: WebSocket client for long connection
        _ws_thread: Thread running WebSocket client
        _processed_message_ids: Ordered deduplication cache
        _loop: Async event loop reference
    """

    name = "feishu"

    def __init__(self, config: FeishuConfig, bus: MessageBus):
        """Initialize Feishu Channel.

        Args:
            config: FeishuConfig configuration object
            bus: MessageBus instance
        """
        super().__init__(config, bus)
        self.config: FeishuConfig = config
        self._client: Any = None
        self._ws_client: Any = None
        self._ws_thread: Optional[threading.Thread] = None
        self._processed_message_ids: OrderedDict[str, None] = OrderedDict()
        self._loop: Optional[asyncio.AbstractEventLoop] = None
        self._streaming_message_ids: dict[str, str] = {}
        self._streaming_last_content: dict[str, str] = {}
        self._typing_reaction_ids: dict[str, str] = {}

    async def start(self) -> None:
        """Start the Feishu bot with WebSocket long connection."""
        if not FEISHU_AVAILABLE:
            logger.error("Feishu SDK not installed. Run: pip install lark-oapi")
            return

        # 降噪：屏蔽 Lark SDK 对未注册事件（如 reaction/message_read）的错误日志刷屏
        for sdk_logger_name in ("lark_oapi", "lark_oapi.ws", "lark_oapi.ws.client"):
            sdk_logger = logging.getLogger(sdk_logger_name)
            sdk_logger.setLevel(logging.CRITICAL)
            sdk_logger.propagate = False

        if not self.config.app_id or not self.config.app_secret:
            logger.error("Feishu app_id and app_secret not configured")
            return

        self._running = True
        self._loop = asyncio.get_running_loop()

        # Create Lark client for sending messages
        self._client = lark.Client.builder() \
            .app_id(self.config.app_id) \
            .app_secret(self.config.app_secret) \
            .log_level(lark.LogLevel.ERROR) \
            .build()

        # Create event handler (only register message receive)
        event_handler = lark.EventDispatcherHandler.builder(
            self.config.encrypt_key or "",
            self.config.verification_token or "",
        ).register_p2_im_message_receive_v1(
            self._on_message_sync
        ).build()

        # Create WebSocket client for long connection
        self._ws_client = lark.ws.Client(
            self.config.app_id,
            self.config.app_secret,
            event_handler=event_handler,
            log_level=lark.LogLevel.ERROR
        )

        # Start WebSocket client in a separate thread with reconnect loop
        def run_ws() -> None:
            import time
            import asyncio
            while self._running:
                try:
                    # 关键修复：lark_oapi/ws/client.py 在模块级别调用 asyncio.get_event_loop()
                    # 并保存在模块变量中。我们需要替换这个变量为新线程的事件循环
                    import lark_oapi.ws.client as ws_client_module
                    
                    # 创建新的事件循环并替换模块变量
                    new_loop = asyncio.new_event_loop()
                    asyncio.set_event_loop(new_loop)
                    ws_client_module.loop = new_loop
                    
                    # 现在可以安全地调用 start()
                    self._ws_client.start()
                except Exception as e:
                    logger.warning(f"Feishu WebSocket error: {e}")
                if self._running:
                    time.sleep(5)

        self._ws_thread = threading.Thread(target=run_ws, daemon=True)
        self._ws_thread.start()

        logger.info("Feishu bot started with WebSocket long connection")
        logger.info("No public IP required - using WebSocket to receive events")

        # Keep running until stopped
        while self._running:
            await asyncio.sleep(1)

    async def stop(self) -> None:
        """Stop the Feishu bot."""
        self._running = False
        if self._ws_client:
            try:
                self._ws_client.stop()
            except Exception as e:
                logger.warning(f"Error stopping WebSocket client: {e}")
        logger.info("Feishu bot stopped")

    # Regex patterns for markdown conversion
    _TABLE_RE = re.compile(
        r"((?:^[ \t]*\|.+\|[ \t]*\n)(?:^[ \t]*\|[-:\s|]+\|[ \t]*\n)(?:^[ \t]*\|.+\|[ \t]*\n?)+)",
        re.MULTILINE,
    )
    _HEADING_RE = re.compile(r"^(#{1,6})\s+(.+)$", re.MULTILINE)
    _CODE_BLOCK_RE = re.compile(r"(```[\s\S]*?```)", re.MULTILINE)

    @staticmethod
    def _parse_md_table(table_text: str) -> Optional[dict]:
        """Parse a markdown table into a Feishu table element."""
        lines = [l.strip() for l in table_text.strip().split("\n") if l.strip()]
        if len(lines) < 3:
            return None

        def split_line(l: str) -> list[str]:
            return [c.strip() for c in l.strip("|").split("|")]

        headers = split_line(lines[0])
        rows = [split_line(l) for l in lines[2:]]
        columns = [
            {"tag": "column", "name": f"c{i}", "display_name": h, "width": "auto"}
            for i, h in enumerate(headers)
        ]
        return {
            "tag": "table",
            "page_size": len(rows) + 1,
            "columns": columns,
            "rows": [
                {f"c{i}": r[i] if i < len(r) else "" for i in range(len(headers))}
                for r in rows
            ],
        }

    def _build_card_elements(self, content: str) -> list[dict]:
        """Split content into div/markdown + table elements for Feishu card."""
        elements: list[dict] = []
        last_end = 0

        for m in self._TABLE_RE.finditer(content):
            before = content[last_end:m.start()]
            if before.strip():
                elements.extend(self._split_headings(before))
            table = self._parse_md_table(m.group(1))
            elements.append(table or {"tag": "markdown", "content": m.group(1)})
            last_end = m.end()

        remaining = content[last_end:]
        if remaining.strip():
            elements.extend(self._split_headings(remaining))

        return elements or [{"tag": "markdown", "content": content}]

    def _split_headings(self, content: str) -> list[dict]:
        """Split content by headings, converting headings to div elements."""
        protected = content
        code_blocks: list[str] = []

        for m in self._CODE_BLOCK_RE.finditer(content):
            code_blocks.append(m.group(1))
            protected = protected.replace(m.group(1), f"\x00CODE{len(code_blocks)-1}\x00", 1)

        elements: list[dict] = []
        last_end = 0

        for m in self._HEADING_RE.finditer(protected):
            before = protected[last_end:m.start()].strip()
            if before:
                elements.append({"tag": "markdown", "content": before})
            text = m.group(2).strip()
            elements.append({
                "tag": "div",
                "text": {
                    "tag": "lark_md",
                    "content": f"**{text}**",
                },
            })
            last_end = m.end()

        remaining = protected[last_end:].strip()
        if remaining:
            elements.append({"tag": "markdown", "content": remaining})

        # Restore code blocks
        for i, cb in enumerate(code_blocks):
            for el in elements:
                if el.get("tag") == "markdown":
                    el["content"] = el["content"].replace(f"\x00CODE{i}\x00", cb)

        return elements or [{"tag": "markdown", "content": content}]

    # File type mappings
    _IMAGE_EXTS = {".png", ".jpg", ".jpeg", ".gif", ".bmp", ".webp", ".ico", ".tiff", ".tif"}
    _AUDIO_EXTS = {".opus"}
    _FILE_TYPE_MAP = {
        ".opus": "opus", ".mp4": "mp4", ".pdf": "pdf", ".doc": "doc", ".docx": "doc",
        ".xls": "xls", ".xlsx": "xls", ".ppt": "ppt", ".pptx": "ppt",
    }

    def _upload_image_sync(self, file_path: str) -> Optional[str]:
        """Upload an image to Feishu and return the image_key."""
        try:
            with open(file_path, "rb") as f:
                request = CreateImageRequest.builder() \
                    .request_body(
                        CreateImageRequestBody.builder()
                        .image_type("message")
                        .image(f)
                        .build()
                    ).build()
                response = self._client.im.v1.image.create(request)
                if response.success():
                    image_key = response.data.image_key
                    logger.debug(f"Uploaded image {os.path.basename(file_path)}: {image_key}")
                    return image_key
                else:
                    logger.error(f"Failed to upload image: code={response.code}, msg={response.msg}")
                    return None
        except Exception as e:
            logger.error(f"Error uploading image {file_path}: {e}")
            return None

    def _upload_file_sync(self, file_path: str) -> Optional[str]:
        """Upload a file to Feishu and return the file_key."""
        ext = os.path.splitext(file_path)[1].lower()
        file_type = self._FILE_TYPE_MAP.get(ext, "stream")
        file_name = os.path.basename(file_path)
        try:
            with open(file_path, "rb") as f:
                request = CreateFileRequest.builder() \
                    .request_body(
                        CreateFileRequestBody.builder()
                        .file_type(file_type)
                        .file_name(file_name)
                        .file(f)
                        .build()
                    ).build()
                response = self._client.im.v1.file.create(request)
                if response.success():
                    file_key = response.data.file_key
                    logger.debug(f"Uploaded file {file_name}: {file_key}")
                    return file_key
                else:
                    logger.error(f"Failed to upload file: code={response.code}, msg={response.msg}")
                    return None
        except Exception as e:
            logger.error(f"Error uploading file {file_path}: {e}")
            return None

    def _download_image_sync(self, message_id: str, image_key: str) -> tuple[Optional[bytes], Optional[str]]:
        """Download an image from Feishu message."""
        try:
            request = GetMessageResourceRequest.builder() \
                .message_id(message_id) \
                .file_key(image_key) \
                .type("image") \
                .build()
            response = self._client.im.v1.message_resource.get(request)
            if response.success():
                file_data = response.file
                if hasattr(file_data, 'read'):
                    file_data = file_data.read()
                return file_data, response.file_name
            else:
                logger.error(f"Failed to download image: code={response.code}, msg={response.msg}")
                return None, None
        except Exception as e:
            logger.error(f"Error downloading image {image_key}: {e}")
            return None, None

    def _download_file_sync(
        self, message_id: str, file_key: str, resource_type: str = "file"
    ) -> tuple[Optional[bytes], Optional[str]]:
        """Download a file/audio/media from a Feishu message."""
        try:
            request = GetMessageResourceRequest.builder() \
                .message_id(message_id) \
                .file_key(file_key) \
                .type(resource_type) \
                .build()
            response = self._client.im.v1.message_resource.get(request)
            if response.success():
                file_data = response.file
                if hasattr(file_data, "read"):
                    file_data = file_data.read()
                return file_data, response.file_name
            else:
                logger.error(f"Failed to download {resource_type}: code={response.code}, msg={response.msg}")
                return None, None
        except Exception as e:
            logger.error(f"Error downloading {resource_type} {file_key}: {e}")
            return None, None

    async def _download_and_save_media(
        self,
        msg_type: str,
        content_json: dict,
        message_id: Optional[str] = None
    ) -> tuple[Optional[str], str]:
        """Download media from Feishu and save to local disk.

        Args:
            msg_type: Message type (image, audio, file, media)
            content_json: Parsed message content
            message_id: Feishu message ID

        Returns:
            (file_path, content_text) - file_path is None if download failed
        """
        loop = asyncio.get_running_loop()
        media_dir = Path.home() / ".iflow-bot" / "media"
        media_dir.mkdir(parents=True, exist_ok=True)

        data: Optional[bytes] = None
        filename: Optional[str] = None

        if msg_type == "image":
            image_key = content_json.get("image_key")
            if image_key and message_id:
                data, filename = await loop.run_in_executor(
                    None, self._download_image_sync, message_id, image_key
                )
                if not filename:
                    filename = f"{image_key[:16]}.jpg"

        elif msg_type in ("audio", "file", "media"):
            file_key = content_json.get("file_key")
            if file_key and message_id:
                data, filename = await loop.run_in_executor(
                    None, self._download_file_sync, message_id, file_key, msg_type
                )
                if not filename:
                    ext = {"audio": ".opus", "media": ".mp4"}.get(msg_type, "")
                    filename = f"{file_key[:16]}{ext}"

        if data and filename:
            file_path = media_dir / filename
            file_path.write_bytes(data)
            logger.debug(f"Downloaded {msg_type} to {file_path}")
            return str(file_path), f"[{msg_type}: {filename}]"

        return None, f"[{msg_type}: download failed]"

    def _send_message_sync(
        self, receive_id_type: str, receive_id: str, msg_type: str, content: str
    ) -> Optional[str]:
        """Send a single message (text/image/file/interactive) synchronously."""
        try:
            request = CreateMessageRequest.builder() \
                .receive_id_type(receive_id_type) \
                .request_body(
                    CreateMessageRequestBody.builder()
                    .receive_id(receive_id)
                    .msg_type(msg_type)
                    .content(content)
                    .build()
                ).build()
            response = self._client.im.v1.message.create(request)
            if not response.success():
                logger.error(
                    f"Failed to send Feishu {msg_type} message: code={response.code}, "
                    f"msg={response.msg}, log_id={response.get_log_id()}"
                )
                return None

            message_id = None
            data = getattr(response, "data", None)
            if data is not None:
                message_id = getattr(data, "message_id", None)
                if message_id is None and isinstance(data, dict):
                    message_id = data.get("message_id")

            logger.debug(f"Feishu {msg_type} message sent to {receive_id}")
            return message_id
        except Exception as e:
            logger.error(f"Error sending Feishu {msg_type} message: {e}")
            return None

    def _patch_message_sync(self, message_id: str, content: str) -> bool:
        """Patch an existing Feishu message content synchronously."""
        try:
            request = PatchMessageRequest.builder() \
                .message_id(message_id) \
                .request_body(
                    PatchMessageRequestBody.builder()
                    .content(content)
                    .build()
                ).build()
            response = self._client.im.v1.message.patch(request)
            if not response.success():
                logger.error(
                    f"Failed to patch Feishu message: code={response.code}, "
                    f"msg={response.msg}, log_id={response.get_log_id()}"
                )
                return False
            return True
        except Exception as e:
            logger.error(f"Error patching Feishu message {message_id}: {e}")
            return False

    async def _handle_streaming_message(
        self, msg: OutboundMessage, receive_id_type: str
    ) -> None:
        """Handle Feishu streaming message by create+patch interactive card."""
        stream_key = msg.chat_id
        loop = asyncio.get_running_loop()

        if msg.metadata.get("_streaming_end"):
            source_message_id = msg.metadata.get("reply_to_id")
            if source_message_id:
                await self._remove_typing_reaction(source_message_id)
            self._streaming_message_ids.pop(stream_key, None)
            self._streaming_last_content.pop(stream_key, None)
            logger.debug(f"Feishu streaming ended: {stream_key}")
            return

        content = (msg.content or "").strip()
        if not content:
            return

        if self._streaming_last_content.get(stream_key) == content:
            return

        source_message_id = msg.metadata.get("reply_to_id")
        if source_message_id:
            await self._remove_typing_reaction(source_message_id)

        card_content = json.dumps({
            "config": {"wide_screen_mode": True},
            "elements": self._build_card_elements(content),
        }, ensure_ascii=False)

        message_id = self._streaming_message_ids.get(stream_key)
        if message_id:
            patched = await loop.run_in_executor(
                None, self._patch_message_sync, message_id, card_content
            )
            if patched:
                self._streaming_last_content[stream_key] = content
                logger.debug(f"Feishu streaming patched: {stream_key}")
                return
            self._streaming_message_ids.pop(stream_key, None)
            logger.warning(f"Feishu streaming patch failed, recreating: {stream_key}")

        created_id = await loop.run_in_executor(
            None, self._send_message_sync,
            receive_id_type, msg.chat_id, "interactive", card_content
        )
        if created_id:
            self._streaming_message_ids[stream_key] = created_id
            self._streaming_last_content[stream_key] = content
            logger.info(f"Feishu streaming placeholder sent: {stream_key}")
        else:
            logger.error(f"Feishu streaming placeholder send failed: {stream_key}")

    def _add_reaction_sync(self, message_id: str, emoji_type: str) -> Optional[str]:
        """Sync helper for adding reaction (runs in thread pool)."""
        try:
            request = CreateMessageReactionRequest.builder() \
                .message_id(message_id) \
                .request_body(
                    CreateMessageReactionRequestBody.builder()
                    .reaction_type(Emoji.builder().emoji_type(emoji_type).build())
                    .build()
                ).build()

            response = self._client.im.v1.message_reaction.create(request)

            if not response.success():
                logger.warning(f"Failed to add reaction: code={response.code}, msg={response.msg}")
                return None

            reaction_id = None
            data = getattr(response, "data", None)
            if data is not None:
                reaction_id = getattr(data, "reaction_id", None)
                if reaction_id is None and isinstance(data, dict):
                    reaction_id = data.get("reaction_id")
            logger.debug(f"Added {emoji_type} reaction to message {message_id}")
            return reaction_id
        except Exception as e:
            logger.warning(f"Error adding reaction: {e}")
            return None

    def _delete_reaction_sync(self, message_id: str, reaction_id: str) -> bool:
        """Sync helper for deleting reaction (runs in thread pool)."""
        try:
            request = DeleteMessageReactionRequest.builder() \
                .message_id(message_id) \
                .reaction_id(reaction_id) \
                .build()
            response = self._client.im.v1.message_reaction.delete(request)
            if not response.success():
                logger.warning(
                    f"Failed to delete reaction: code={response.code}, msg={response.msg}"
                )
                return False
            logger.debug(f"Deleted reaction from message {message_id}")
            return True
        except Exception as e:
            logger.warning(f"Error deleting reaction: {e}")
            return False

    async def _add_reaction(self, message_id: str, emoji_type: str = "THUMBSUP") -> Optional[str]:
        """Add a reaction emoji to a message (non-blocking).

        Common emoji types: THUMBSUP, OK, EYES, DONE, OnIt, HEART
        """
        if not self._client or not Emoji:
            return None

        loop = asyncio.get_running_loop()
        return await loop.run_in_executor(None, self._add_reaction_sync, message_id, emoji_type)

    async def _remove_typing_reaction(self, source_message_id: str) -> None:
        """Remove the pending typing reaction from a source message."""
        reaction_id = self._typing_reaction_ids.pop(source_message_id, None)
        if not reaction_id:
            return

        loop = asyncio.get_running_loop()
        deleted = await loop.run_in_executor(
            None, self._delete_reaction_sync, source_message_id, reaction_id
        )
        if deleted:
            logger.debug(f"Typing reaction cleared: {source_message_id}")

    async def send(self, msg: OutboundMessage) -> None:
        """Send a message through Feishu, including media (images/files) if present.

        Supports:
        - Text messages (converted to interactive cards)
        - Image messages
        - File messages
        - Audio messages

        Args:
            msg: OutboundMessage to send
        """
        if not self._client:
            logger.warning("Feishu client not initialized")
            return

        try:
            # Determine receive_id_type based on chat_id prefix
            receive_id_type = "chat_id" if msg.chat_id.startswith("oc_") else "open_id"
            loop = asyncio.get_running_loop()

            if msg.metadata.get("_streaming") or msg.metadata.get("_streaming_end"):
                await self._handle_streaming_message(msg, receive_id_type)
                return

            if msg.metadata.get("_progress"):
                return

            source_message_id = msg.metadata.get("reply_to_id")
            if source_message_id:
                await self._remove_typing_reaction(source_message_id)

            self._streaming_message_ids.pop(msg.chat_id, None)
            self._streaming_last_content.pop(msg.chat_id, None)

            # Handle media files first
            media = getattr(msg, 'media', None) or []
            for file_path in media:
                if not os.path.isfile(file_path):
                    logger.warning(f"Media file not found: {file_path}")
                    continue
                ext = os.path.splitext(file_path)[1].lower()
                if ext in self._IMAGE_EXTS:
                    key = await loop.run_in_executor(None, self._upload_image_sync, file_path)
                    if key:
                        await loop.run_in_executor(
                            None, self._send_message_sync,
                            receive_id_type, msg.chat_id, "image",
                            json.dumps({"image_key": key}, ensure_ascii=False),
                        )
                else:
                    key = await loop.run_in_executor(None, self._upload_file_sync, file_path)
                    if key:
                        media_type = "audio" if ext in self._AUDIO_EXTS else "file"
                        await loop.run_in_executor(
                            None, self._send_message_sync,
                            receive_id_type, msg.chat_id, media_type,
                            json.dumps({"file_key": key}, ensure_ascii=False),
                        )

            # Send text content as interactive card
            if msg.content and msg.content.strip():
                card = {
                    "config": {"wide_screen_mode": True},
                    "elements": self._build_card_elements(msg.content)
                }
                await loop.run_in_executor(
                    None, self._send_message_sync,
                    receive_id_type, msg.chat_id, "interactive",
                    json.dumps(card, ensure_ascii=False),
                )

        except Exception as e:
            logger.error(f"Error sending Feishu message: {e}")

    def _on_message_sync(self, data: "P2ImMessageReceiveV1") -> None:
        """Sync handler for incoming messages (called from WebSocket thread).

        Schedules async handling in the main event loop.
        """
        if self._loop and self._loop.is_running():
            asyncio.run_coroutine_threadsafe(self._on_message(data), self._loop)

    async def _on_message(self, data: "P2ImMessageReceiveV1") -> None:
        """Handle incoming message from Feishu.

        Processes message content, downloads media if present,
        and publishes to message bus.
        """
        try:
            event = data.event
            message = event.message
            sender = event.sender

            # Deduplication check
            message_id = message.message_id
            if message_id in self._processed_message_ids:
                return
            self._processed_message_ids[message_id] = None

            # Trim cache (keep last 1000 messages)
            while len(self._processed_message_ids) > 1000:
                self._processed_message_ids.popitem(last=False)

            # Skip bot messages
            if sender.sender_type == "bot":
                return

            sender_id = sender.sender_id.open_id if sender.sender_id else "unknown"
            chat_id = message.chat_id
            chat_type = message.chat_type
            msg_type = message.message_type

            # 秒回“敲键盘”反应，作为处理中提示
            typing_reaction_id = await self._add_reaction(message_id, "OnIt")
            if typing_reaction_id:
                self._typing_reaction_ids[message_id] = typing_reaction_id

            # Parse content
            content_parts: list[str] = []
            media_paths: list[str] = []

            try:
                content_json = json.loads(message.content) if message.content else {}
            except json.JSONDecodeError:
                content_json = {}

            if msg_type == "text":
                text = content_json.get("text", "")
                if text:
                    content_parts.append(text)

            elif msg_type == "post":
                text = _extract_post_text(content_json)
                if text:
                    content_parts.append(text)

            elif msg_type in ("image", "audio", "file", "media"):
                file_path, content_text = await self._download_and_save_media(
                    msg_type, content_json, message_id
                )
                if file_path:
                    media_paths.append(file_path)
                content_parts.append(content_text)

            elif msg_type in ("share_chat", "share_user", "interactive",
                            "share_calendar_event", "system", "merge_forward"):
                text = _extract_share_card_content(content_json, msg_type)
                if text:
                    content_parts.append(text)

            else:
                content_parts.append(MSG_TYPE_MAP.get(msg_type, f"[{msg_type}]"))

            content = "\n".join(content_parts) if content_parts else ""

            if not content and not media_paths:
                return

            # Determine reply target: group chat uses chat_id, private uses sender_id
            reply_to = chat_id if chat_type == "group" else sender_id

            logger.info(
                "Feishu inbound accepted: message_id=%s chat_type=%s msg_type=%s sender=%s reply_to=%s content_len=%d media=%d",
                message_id,
                chat_type,
                msg_type,
                sender_id,
                reply_to,
                len(content),
                len(media_paths),
            )

            # Forward to message bus
            await self._handle_message(
                sender_id=sender_id,
                chat_id=reply_to,
                content=content,
                media=media_paths,
                metadata={
                    "message_id": message_id,
                    "chat_type": chat_type,
                    "msg_type": msg_type,
                }
            )

        except Exception as e:
            logger.error(f"Error processing Feishu message: {e}")



"""DingTalk (钉钉) channel implementation using Stream Mode.

使用 dingtalk-stream SDK 通过 WebSocket 接收消息，
使用 HTTP API 发送消息。无需公网 IP 或 webhook 配置。

支持 AI Card 流式输出，实现打字机效果。
"""

import asyncio
import json
import logging
import time
import uuid
from typing import Any, Dict, Optional, Set

from iflow_bot.bus.events import OutboundMessage
from iflow_bot.bus.queue import MessageBus
from iflow_bot.channels.base import BaseChannel
from iflow_bot.channels.manager import register_channel
from iflow_bot.config.schema import DingTalkConfig

try:
    import httpx
    HTTPX_AVAILABLE = True
except ImportError:
    HTTPX_AVAILABLE = False
    httpx = None  # type: ignore

try:
    from dingtalk_stream import (
        DingTalkStreamClient,
        Credential,
        CallbackHandler,
        CallbackMessage,
        AckMessage,
    )
    from dingtalk_stream.chatbot import ChatbotMessage
    DINGTALK_AVAILABLE = True
except ImportError:
    DINGTALK_AVAILABLE = False
    CallbackHandler = object  # type: ignore
    CallbackMessage = None  # type: ignore
    AckMessage = None  # type: ignore
    ChatbotMessage = None  # type: ignore


logger = logging.getLogger(__name__)

# AI Card 状态常量
class AICardStatus:
    PROCESSING = "1"
    INPUTING = "2"
    FINISHED = "3"
    FAILED = "5"


class AICardInstance:
    """AI Card 实例，用于跟踪流式输出状态。"""
    
    def __init__(
        self,
        card_instance_id: str,
        access_token: str,
        conversation_id: str,
        config: DingTalkConfig,
    ):
        self.card_instance_id = card_instance_id
        self.access_token = access_token
        self.conversation_id = conversation_id
        self.config = config
        self.created_at = time.time()
        self.last_updated = time.time()
        self.state = AICardStatus.PROCESSING


def is_card_in_terminal_state(state: str) -> bool:
    """检查 Card 是否处于终态。"""
    return state in (AICardStatus.FINISHED, AICardStatus.FAILED)


class DingTalkHandler(CallbackHandler):
    """钉钉 Stream SDK 回调处理器。"""

    def __init__(self, channel: "DingTalkChannel"):
        super().__init__()
        self.channel = channel

    async def process(self, message: CallbackMessage):
        """处理入站的 stream 消息。"""
        try:
            chatbot_msg = ChatbotMessage.from_dict(message.data)

            content = ""
            if chatbot_msg.text:
                content = chatbot_msg.text.content.strip()
            if not content:
                content = message.data.get("text", {}).get("content", "").strip()

            if not content:
                logger.warning(
                    f"[{self.channel.name}] Received empty or unsupported message type: "
                    f"{chatbot_msg.message_type}"
                )
                return AckMessage.STATUS_OK, "OK"

            sender_id = chatbot_msg.sender_staff_id or chatbot_msg.sender_id
            sender_name = chatbot_msg.sender_nick or "Unknown"
            
            # 判断是否群聊
            is_group = chatbot_msg.conversation_type == "2"
            chat_id = chatbot_msg.conversation_id if is_group else sender_id

            logger.info(
                f"[{self.channel.name}] Received message from {sender_name} "
                f"({sender_id}) in {'group' if is_group else 'private'}: {content[:50]}..."
            )

            task = asyncio.create_task(
                self.channel._on_message(content, sender_id, sender_name, chat_id, is_group)
            )
            self.channel._background_tasks.add(task)
            task.add_done_callback(self.channel._background_tasks.discard)

            return AckMessage.STATUS_OK, "OK"

        except Exception as e:
            logger.error(f"[{self.channel.name}] Error processing message: {e}")
            return AckMessage.STATUS_OK, "Error"


@register_channel("dingtalk")
class DingTalkChannel(BaseChannel):
    """钉钉 Channel - 使用 Stream Mode，支持 AI Card 流式输出。"""

    name = "dingtalk"
    
    # 支持流式输出的渠道标识
    supports_streaming = True

    def __init__(self, config: DingTalkConfig, bus: MessageBus):
        super().__init__(config, bus)
        self.config: DingTalkConfig = config
        self._client: Any = None
        self._http: Optional[httpx.AsyncClient] = None

        # Access Token 管理
        self._access_token: Optional[str] = None
        self._token_expiry: float = 0

        # AI Card 管理
        self._ai_cards: Dict[str, AICardInstance] = {}
        self._active_cards_by_target: Dict[str, str] = {}

        # 流式消息缓冲
        self._streaming_buffers: Dict[str, str] = {}
        self._streaming_card_ids: Dict[str, str] = {}
        self._streaming_end: Set[str] = set()
        self._streaming_last_sent_content: Dict[str, str] = {}
        self._streaming_last_sent_at: Dict[str, float] = {}

        # 后台任务
        self._background_tasks: Set[asyncio.Task] = set()

    async def start(self) -> None:
        """启动钉钉 Bot (Stream Mode)。"""
        if not DINGTALK_AVAILABLE:
            logger.error(
                f"[{self.name}] DingTalk Stream SDK not installed. "
                "Run: pip install dingtalk-stream"
            )
            return

        if not HTTPX_AVAILABLE:
            logger.error(f"[{self.name}] httpx not installed. Run: pip install httpx")
            return

        if not self.config.client_id or not self.config.client_secret:
            logger.error(f"[{self.name}] client_id and client_secret not configured")
            return

        self._running = True
        self._http = httpx.AsyncClient()

        logger.info(
            f"[{self.name}] Initializing DingTalk Stream Client with Client ID: "
            f"{self.config.client_id[:8]}..."
        )

        credential = Credential(self.config.client_id, self.config.client_secret)
        self._client = DingTalkStreamClient(credential)

        handler = DingTalkHandler(self)
        self._client.register_callback_handler(ChatbotMessage.TOPIC, handler)

        logger.info(f"[{self.name}] DingTalk bot started with Stream Mode")

        while self._running:
            try:
                await self._client.start()
            except Exception as e:
                logger.warning(f"[{self.name}] DingTalk stream error: {e}")
            if self._running:
                logger.info(f"[{self.name}] Reconnecting in 5 seconds...")
                await asyncio.sleep(5)

    async def stop(self) -> None:
        """停止钉钉 Bot。"""
        self._running = False

        if self._http:
            await self._http.aclose()
            self._http = None

        for task in self._background_tasks:
            task.cancel()
        self._background_tasks.clear()
        self._streaming_last_sent_content.clear()
        self._streaming_last_sent_at.clear()

        logger.info(f"[{self.name}] DingTalk bot stopped")

    async def _get_access_token(self) -> Optional[str]:
        """获取或刷新 Access Token。"""
        if self._access_token and time.time() < self._token_expiry:
            return self._access_token

        url = "https://api.dingtalk.com/v1.0/oauth2/accessToken"
        data = {
            "appKey": self.config.client_id,
            "appSecret": self.config.client_secret,
        }

        if not self._http:
            return None

        try:
            resp = await self._http.post(url, json=data)
            resp.raise_for_status()
            res_data = resp.json()
            self._access_token = res_data.get("accessToken")
            self._token_expiry = time.time() + int(res_data.get("expireIn", 7200)) - 60
            return self._access_token
        except Exception as e:
            logger.error(f"[{self.name}] Failed to get access token: {e}")
            return None

    def _get_target_key(self, chat_id: str) -> str:
        """获取目标唯一标识。"""
        return f"{self.config.client_id}:{chat_id}"

    async def _create_ai_card(self, chat_id: str, is_group: bool = False) -> Optional[AICardInstance]:
        """创建 AI Card 实例。"""
        token = await self._get_access_token()
        if not token:
            return None

        card_template_id = getattr(self.config, 'card_template_id', None)
        if not card_template_id:
            logger.debug(f"[{self.name}] No card_template_id configured, using markdown")
            return None

        card_instance_id = f"card_{uuid.uuid4().hex[:24]}"
        
        # 构建 openSpaceId
        if is_group:
            open_space_id = f"dtv1.card//IM_GROUP.{chat_id}"
        else:
            open_space_id = f"dtv1.card//IM_ROBOT.{chat_id}"

        create_body = {
            "cardTemplateId": card_template_id,
            "outTrackId": card_instance_id,
            "cardData": {"cardParamMap": {}},
            "callbackType": "STREAM",
            "imGroupOpenSpaceModel": {"supportForward": True},
            "imRobotOpenSpaceModel": {"supportForward": True},
            "openSpaceId": open_space_id,
            "userIdType": 1,
        }

        if is_group:
            robot_code = getattr(self.config, 'robot_code', None) or self.config.client_id
            create_body["imGroupOpenDeliverModel"] = {"robotCode": robot_code}
        else:
            create_body["imRobotOpenDeliverModel"] = {"spaceType": "IM_ROBOT"}

        try:
            resp = await self._http.post(
                "https://api.dingtalk.com/v1.0/card/instances/createAndDeliver",
                json=create_body,
                headers={
                    "x-acs-dingtalk-access-token": token,
                    "Content-Type": "application/json",
                },
            )
            resp.raise_for_status()

            card = AICardInstance(
                card_instance_id=card_instance_id,
                access_token=token,
                conversation_id=chat_id,
                config=self.config,
            )
            self._ai_cards[card_instance_id] = card
            
            target_key = self._get_target_key(chat_id)
            self._active_cards_by_target[target_key] = card_instance_id

            logger.debug(f"[{self.name}] Created AI Card: {card_instance_id}")
            return card

        except Exception as e:
            logger.error(f"[{self.name}] Failed to create AI Card: {e}")
            return None

    async def _stream_ai_card(self, card: AICardInstance, content: str, finished: bool = False) -> bool:
        """流式更新 AI Card 内容。"""
        if card.state == AICardStatus.FINISHED:
            return False

        # Token 刷新检查 (90分钟阈值)
        token_age = time.time() - card.created_at
        if token_age > 90 * 60:
            new_token = await self._get_access_token()
            if new_token:
                card.access_token = new_token

        card_template_key = getattr(self.config, 'card_template_key', 'content')
        
        stream_body = {
            "outTrackId": card.card_instance_id,
            "guid": uuid.uuid4().hex,
            "key": card_template_key,
            "content": content,
            "isFull": True,
            "isFinalize": finished,
            "isError": False,
        }

        try:
            resp = await self._http.put(
                "https://api.dingtalk.com/v1.0/card/streaming",
                json=stream_body,
                headers={
                    "x-acs-dingtalk-access-token": card.access_token,
                    "Content-Type": "application/json",
                },
            )
            if resp.status_code >= 400:
                logger.error(f"[{self.name}] AI Card streaming error: status={resp.status_code}, body={resp.text[:500]}")
            resp.raise_for_status()

            card.last_updated = time.time()
            if finished:
                card.state = AICardStatus.FINISHED
            elif card.state == AICardStatus.PROCESSING:
                card.state = AICardStatus.INPUTING

            return True

        except Exception as e:
            logger.error(f"[{self.name}] AI Card streaming failed: {e}")
            card.state = AICardStatus.FAILED
            return False

    async def _send_markdown(self, chat_id: str, content: str, is_group: bool = False) -> bool:
        """发送 Markdown 消息（非 Card 模式）。"""
        token = await self._get_access_token()
        if not token:
            return False

        robot_code = getattr(self.config, 'robot_code', None) or self.config.client_id

        if is_group:
            url = "https://api.dingtalk.com/v1.0/robot/groupMessages/send"
            data = {
                "robotCode": robot_code,
                "openConversationId": chat_id,
                "msgKey": "sampleMarkdown",
                "msgParam": json.dumps({
                    "title": "iFlow Bot",
                    "text": content,
                }, ensure_ascii=False),
            }
        else:
            url = "https://api.dingtalk.com/v1.0/robot/oToMessages/batchSend"
            data = {
                "robotCode": robot_code,
                "userIds": [chat_id],
                "msgKey": "sampleMarkdown",
                "msgParam": json.dumps({
                    "title": "iFlow Bot Reply",
                    "text": content,
                }, ensure_ascii=False),
            }

        try:
            resp = await self._http.post(
                url,
                json=data,
                headers={"x-acs-dingtalk-access-token": token},
            )
            return resp.status_code == 200
        except Exception as e:
            logger.error(f"[{self.name}] Send markdown failed: {e}")
            return False

    async def send(self, msg: OutboundMessage) -> None:
        """发送消息（非流式）。"""
        is_group = msg.metadata.get("is_group", False) if msg.metadata else False
        await self._send_markdown(msg.chat_id, msg.content, is_group)

    async def start_streaming(self, chat_id: str) -> None:
        """开始流式输出，立即创建 AI Card。

        在消息处理开始时调用，确保卡片秒回。

        Args:
            chat_id: 聊天 ID
        """
        target_key = self._get_target_key(chat_id)
        
        # 初始化缓冲区
        if target_key not in self._streaming_buffers:
            self._streaming_buffers[target_key] = ""
            
            # 立即创建 AI Card
            is_group = chat_id.startswith("cid")
            card = await self._create_ai_card(chat_id, is_group)
            if card:
                self._streaming_card_ids[target_key] = card.card_instance_id
                logger.debug(f"[{self.name}] Created AI Card for streaming: {card.card_instance_id}")

    async def handle_streaming_chunk(self, chat_id: str, chunk: str, is_final: bool = False) -> None:
        """处理流式输出的 chunk。

        由 AgentLoop 调用，实现打字机效果。

        注意：chunk 参数已经是累积后的完整内容（由 AgentLoop 累积），
        这里直接使用，不再重复累积。

        Args:
            chat_id: 聊天 ID
            chunk: 累积后的完整文本内容
            is_final: 是否是最终消息
        """
        target_key = self._get_target_key(chat_id)
        
        # 确保缓冲区已初始化（兼容没有调用 start_streaming 的情况）
        if target_key not in self._streaming_buffers:
            await self.start_streaming(chat_id)

        # 直接使用传入的内容（已经是累积后的）
        full_content = chunk

        # 获取 Card
        card_id = self._streaming_card_ids.get(target_key)
        card = self._ai_cards.get(card_id) if card_id else None

        if card and not is_card_in_terminal_state(card.state):
            # 使用 AI Card 流式更新
            await self._stream_ai_card(card, full_content, is_final)
        else:
            # 降级为 Markdown 流式发送（createAndDeliver 失败时仍保持可见输出）
            is_group = chat_id.startswith("cid")
            await self._send_streaming_markdown_fallback(
                chat_id=chat_id,
                target_key=target_key,
                full_content=full_content,
                is_group=is_group,
                is_final=is_final,
            )

        # 清理
        if is_final:
            self._streaming_buffers.pop(target_key, None)
            self._streaming_card_ids.pop(target_key, None)
            self._streaming_end.discard(target_key)
            self._streaming_last_sent_content.pop(target_key, None)
            self._streaming_last_sent_at.pop(target_key, None)

    async def _send_streaming_markdown_fallback(
        self,
        chat_id: str,
        target_key: str,
        full_content: str,
        is_group: bool,
        is_final: bool,
    ) -> None:
        """Fallback streaming when AI Card is unavailable.

        DingTalk markdown messages cannot be edited in-place via current path,
        so we throttle progressive sends to keep output visible without flooding.
        """
        text = (full_content or "").strip()
        if not text:
            return

        last_content = self._streaming_last_sent_content.get(target_key, "")
        now = time.time()
        last_sent_at = self._streaming_last_sent_at.get(target_key, 0.0)

        # Send on first visible chunk, then throttle by time/size delta.
        should_send_progress = (
            last_content == ""
            or (len(text) - len(last_content) >= 80)
            or (now - last_sent_at >= 2.0)
        )

        if is_final:
            # Ensure final full response is visible once.
            if text != last_content:
                await self._send_markdown(chat_id, text, is_group)
                self._streaming_last_sent_content[target_key] = text
                self._streaming_last_sent_at[target_key] = now
            return

        if should_send_progress and text != last_content:
            await self._send_markdown(chat_id, text, is_group)
            self._streaming_last_sent_content[target_key] = text
            self._streaming_last_sent_at[target_key] = now

    async def _on_message(
        self,
        content: str,
        sender_id: str,
        sender_name: str,
        chat_id: str,
        is_group: bool,
    ) -> None:
        """处理入站消息。"""
        try:
            await self._handle_message(
                sender_id=sender_id,
                chat_id=chat_id,
                content=content,
                metadata={
                    "sender_name": sender_name,
                    "platform": "dingtalk",
                    "is_group": is_group,
                },
            )
        except Exception as e:
            logger.error(f"[{self.name}] Error publishing message: {e}")



"""Session manager for iflow-bot.

Manages user sessions across different channels. Each session is stored
as a JSON file tracking metadata like creation time, last active time,
and message count. The actual conversation history is managed by iflow.
"""

import json
from datetime import datetime, timezone
from pathlib import Path
from typing import Optional

from pydantic import BaseModel


class SessionMetadata(BaseModel):
    """Session metadata model."""
    
    key: str
    """Session key in format: channel:chat_id"""
    
    channel: str
    """Channel name (telegram, discord, etc.)"""
    
    chat_id: str
    """Chat/channel/conversation identifier"""
    
    created_at: str
    """ISO format creation timestamp"""
    
    last_active: str
    """ISO format last active timestamp"""
    
    message_count: int = 0
    """Number of messages in this session"""


class SessionManager:
    """Manages user sessions for iflow-bot.
    
    Sessions are stored as JSON files in the workspace/sessions directory.
    Each session tracks metadata for a unique channel+chat_id combination.
    
    The actual conversation history is managed by iflow CLI (stored in
    ~/.iflow/sessions/), while this module tracks iflow-bot specific
    session state.
    
    Example:
        >>> manager = SessionManager("/path/to/workspace")
        >>> key = manager.get_session_key("telegram", "123456")
        >>> manager.create_session(key)
        '/path/to/workspace/sessions/telegram_123456.json'
    """
    
    def __init__(self, workspace_path: str):
        """Initialize the session manager.
        
        Args:
            workspace_path: Path to the workspace directory where sessions
                           will be stored.
        """
        self.workspace_path = Path(workspace_path)
        self.session_dir = self.workspace_path / "sessions"
        self.session_dir.mkdir(parents=True, exist_ok=True)
    
    def get_session_key(self, channel: str, chat_id: str) -> str:
        """Get session key from channel and chat_id.
        
        Args:
            channel: Channel name (e.g., 'telegram', 'discord')
            chat_id: Chat/conversation identifier
            
        Returns:
            Session key in format 'channel:chat_id'
        """
        return f"{channel}:{chat_id}"
    
    def _get_session_filename(self, session_key: str) -> str:
        """Convert session key to filename.
        
        Args:
            session_key: Session key in format 'channel:chat_id'
            
        Returns:
            Filename in format 'channel_chat_id.json'
        """
        # Replace ':' with '_' for filesystem compatibility
        return f"{session_key.replace(':', '_')}.json"
    
    def get_session_file(self, session_key: str) -> Path:
        """Get the path to the session file.
        
        Args:
            session_key: Session key in format 'channel:chat_id'
            
        Returns:
            Path to the session JSON file
        """
        filename = self._get_session_filename(session_key)
        return self.session_dir / filename
    
    def session_exists(self, session_key: str) -> bool:
        """Check if a session exists.
        
        Args:
            session_key: Session key in format 'channel:chat_id'
            
        Returns:
            True if session file exists, False otherwise
        """
        session_file = self.get_session_file(session_key)
        return session_file.exists()
    
    def create_session(self, session_key: str) -> str:
        """Create a new session.
        
        Creates a new session file with initial metadata. If session
        already exists, returns the existing session file path.
        
        Args:
            session_key: Session key in format 'channel:chat_id'
            
        Returns:
            Path to the session file (as string)
        """
        if self.session_exists(session_key):
            return str(self.get_session_file(session_key))
        
        # Parse channel and chat_id from key
        parts = session_key.split(":", 1)
        if len(parts) != 2:
            raise ValueError(f"Invalid session key format: {session_key}")
        
        channel, chat_id = parts
        now = datetime.now(timezone.utc).isoformat()
        
        metadata = SessionMetadata(
            key=session_key,
            channel=channel,
            chat_id=chat_id,
            created_at=now,
            last_active=now,
            message_count=0,
        )
        
        session_file = self.get_session_file(session_key)
        session_file.write_text(
            metadata.model_dump_json(indent=2),
            encoding="utf-8"
        )
        
        return str(session_file)
    
    def get_session(self, session_key: str) -> Optional[SessionMetadata]:
        """Get session metadata.
        
        Args:
            session_key: Session key in format 'channel:chat_id'
            
        Returns:
            SessionMetadata if session exists, None otherwise
        """
        if not self.session_exists(session_key):
            return None
        
        session_file = self.get_session_file(session_key)
        try:
            data = json.loads(session_file.read_text(encoding="utf-8"))
            return SessionMetadata(**data)
        except (json.JSONDecodeError, ValueError):
            return None
    
    def list_sessions(self) -> list[dict]:
        """List all sessions.
        
        Returns:
            List of session metadata dictionaries, sorted by last_active
            (most recent first)
        """
        sessions = []
        
        for session_file in self.session_dir.glob("*.json"):
            try:
                data = json.loads(session_file.read_text(encoding="utf-8"))
                sessions.append(data)
            except (json.JSONDecodeError, ValueError):
                # Skip invalid session files
                continue
        
        # Sort by last_active, most recent first
        sessions.sort(key=lambda s: s.get("last_active", ""), reverse=True)
        
        return sessions
    
    def update_session(
        self,
        session_key: str,
        metadata: Optional[dict] = None,
        increment_count: bool = False,
    ) -> bool:
        """Update session metadata.
        
        Updates the session's last_active timestamp and optionally
        other metadata fields.
        
        Args:
            session_key: Session key in format 'channel:chat_id'
            metadata: Optional dict of additional metadata to update
            increment_count: If True, increment message_count by 1
            
        Returns:
            True if update succeeded, False if session doesn't exist
        """
        session_data = self.get_session(session_key)
        if session_data is None:
            return False
        
        # Update last_active timestamp
        now = datetime.now(timezone.utc).isoformat()
        session_data.last_active = now
        
        # Increment message count if requested
        if increment_count:
            session_data.message_count += 1
        
        # Apply additional metadata updates
        if metadata:
            for key, value in metadata.items():
                if hasattr(session_data, key):
                    setattr(session_data, key, value)
        
        # Write updated metadata
        session_file = self.get_session_file(session_key)
        session_file.write_text(
            session_data.model_dump_json(indent=2),
            encoding="utf-8"
        )
        
        return True
    
    def delete_session(self, session_key: str) -> bool:
        """Delete a session.
        
        Removes the session file. Note: This only removes the iflow-bot
        session metadata. The actual conversation history managed by
        iflow CLI is not affected.
        
        Args:
            session_key: Session key in format 'channel:chat_id'
            
        Returns:
            True if deletion succeeded, False if session didn't exist
        """
        if not self.session_exists(session_key):
            return False
        
        session_file = self.get_session_file(session_key)
        session_file.unlink()
        return True
    
    def get_or_create_session(self, channel: str, chat_id: str) -> str:
        """Get existing session or create a new one.
        
        Convenience method that combines get_session_key and create_session.
        
        Args:
            channel: Channel name (e.g., 'telegram', 'discord')
            chat_id: Chat/conversation identifier
            
        Returns:
            Path to the session file (as string)
        """
        session_key = self.get_session_key(channel, chat_id)
        return self.create_session(session_key)
    
    def touch_session(self, channel: str, chat_id: str) -> bool:
        """Update session's last_active timestamp.
        
        Convenience method that updates a session's activity timestamp
        and increments the message count.
        
        Args:
            channel: Channel name
            chat_id: Chat/conversation identifier
            
        Returns:
            True if session was updated, False if it doesn't exist
        """
        session_key = self.get_session_key(channel, chat_id)
        return self.update_session(session_key, increment_count=True)
    
    def get_sessions_by_channel(self, channel: str) -> list[dict]:
        """Get all sessions for a specific channel.
        
        Args:
            channel: Channel name to filter by
            
        Returns:
            List of session metadata dictionaries for the channel
        """
        all_sessions = self.list_sessions()
        return [s for s in all_sessions if s.get("channel") == channel]
    
    def cleanup_old_sessions(
        self,
        days_old: int = 30,
        dry_run: bool = False,
    ) -> list[str]:
        """Clean up sessions older than specified days.
        
        Args:
            days_old: Number of days after which to consider a session old
            dry_run: If True, only list sessions to be deleted without
                    actually deleting them
                    
        Returns:
            List of session keys that were (or would be) deleted
        """
        cutoff = datetime.now(timezone.utc)
        deleted = []
        
        for session_data in self.list_sessions():
            last_active_str = session_data.get("last_active", "")
            if not last_active_str:
                continue
            
            try:
                last_active = datetime.fromisoformat(
                    last_active_str.replace("Z", "+00:00")
                )
                age_days = (cutoff - last_active.replace(tzinfo=timezone.utc)).days
                
                if age_days > days_old:
                    session_key = session_data.get("key", "")
                    if not dry_run:
                        self.delete_session(session_key)
                    deleted.append(session_key)
            except ValueError:
                continue
        
        return deleted



"""ACP (Agent Communication Protocol) Connector for iflow CLI.

实现 ACP 协议连接器，不依赖官方 SDK，直接使用 WebSocket 进行通信。

ACP 协议基于 JSON-RPC 2.0，使用 WebSocket 作为传输层。

核心方法：
- initialize: 初始化连接，协商协议版本和能力
- session/create: 创建新会话
- session/prompt: 发送消息
- session/update: 接收更新通知
- session/cancel: 取消当前请求

消息类型：
- agent_message_chunk: Agent 响应块
- agent_thought_chunk: Agent 思考过程
- tool_call/tool_call_update: 工具调用
- stop_reason: 任务完成
"""

from __future__ import annotations

import asyncio
import json
import time
import uuid
from dataclasses import dataclass, field
from enum import Enum
from pathlib import Path
from typing import Any, Callable, Coroutine, Optional, Union

from loguru import logger

from iflow_bot.config.loader import DEFAULT_TIMEOUT

try:
    import websockets
    WEBSOCKETS_AVAILABLE = True
except ImportError:
    WEBSOCKETS_AVAILABLE = False
    websockets = None  # type: ignore


class ACPError(Exception):
    """ACP 连接器错误基类。"""
    pass


class ACPConnectionError(ACPError):
    """ACP 连接错误。"""
    pass


class ACPTimeoutError(ACPError):
    """ACP 超时错误。"""
    pass


class StopReason(str, Enum):
    """任务结束原因。"""
    END_TURN = "end_turn"
    MAX_TOKENS = "max_tokens"
    REFUSAL = "refusal"
    CANCELLED = "cancelled"
    ERROR = "error"


@dataclass
class ContentBlock:
    """内容块基类。"""
    type: str = "text"
    text: str = ""


@dataclass
class TextContent(ContentBlock):
    """文本内容块。"""
    type: str = "text"
    text: str = ""


@dataclass
class AgentMessageChunk:
    """Agent 消息块。"""
    text: str = ""
    is_thought: bool = False


@dataclass
class ToolCall:
    """工具调用信息。"""
    tool_call_id: str
    tool_name: str
    status: str = "pending"  # pending, in_progress, completed, failed
    content: list[ContentBlock] = field(default_factory=list)
    args: dict = field(default_factory=dict)
    output: str = ""


@dataclass
class SessionUpdate:
    """会话更新消息。"""
    update_type: str
    data: dict = field(default_factory=dict)


@dataclass
class ACPResponse:
    """ACP 响应结果。"""
    content: str = ""
    thought: str = ""
    tool_calls: list[ToolCall] = field(default_factory=list)
    stop_reason: Optional[StopReason] = None
    error: Optional[str] = None


class ACPClient:
    """
    ACP 协议客户端 - 与 iflow CLI 的 ACP 模式通信。
    
    使用 WebSocket 连接到 iflow --experimental-acp 启动的服务器。
    实现完整的 JSON-RPC 2.0 消息格式。
    
    Example:
        client = ACPClient(host="localhost", port=8090)
        await client.connect()
        await client.initialize()
        session_id = await client.create_session(workspace="/path/to/workspace")
        response = await client.prompt(session_id, "Hello!")
        print(response.content)
    """
    
    PROTOCOL_VERSION = 1
    
    def __init__(
        self,
        host: str = "localhost",
        port: int = 8090,
        timeout: int = DEFAULT_TIMEOUT,
        workspace: Optional[Path] = None,
    ):
        """
        初始化 ACP 客户端。
        
        Args:
            host: ACP 服务器主机地址
            port: ACP 服务器端口
            timeout: 请求超时时间（秒）
            workspace: 工作目录路径
        """
        if not WEBSOCKETS_AVAILABLE:
            raise ImportError("websockets library not installed. Run: pip install websockets")
        
        self.host = host
        self.port = port
        self.timeout = timeout
        self.workspace = workspace
        
        self._ws: Any = None
        self._connected = False
        self._initialized = False
        self._request_id = 0
        self._pending_requests: dict[int, asyncio.Future] = {}
        self._receive_task: Optional[asyncio.Task] = None
        self._message_queue: asyncio.Queue[dict] = asyncio.Queue()
        self._session_queues: dict[str, asyncio.Queue[dict]] = {}
        self._prompt_lock = asyncio.Lock()  # 保证请求发送原子性
        
        # Agent 能力
        self._agent_capabilities: dict = {}
        
        logger.info(f"ACPClient initialized: ws://{host}:{port}/acp")
    
    @property
    def ws_url(self) -> str:
        """获取 WebSocket URL。"""
        return f"ws://{self.host}:{self.port}/acp"
    
    async def connect(self) -> None:
        """连接到 ACP 服务器。"""
        if self._connected:
            return
        
        try:
            self._ws = await websockets.connect(
                self.ws_url,
                ping_interval=30,
                ping_timeout=10,
                close_timeout=5,
            )
            
            self._connected = True
            
            # 启动消息接收任务
            self._receive_task = asyncio.create_task(self._receive_loop())
            
            logger.info(f"ACP connected to {self.ws_url}")
            
            # 等待服务器的就绪信号 "//ready"
            # 然后等待 peer 准备好（大约需要 2-3 秒）
            await asyncio.sleep(3)
            
        except Exception as e:
            raise ACPConnectionError(f"Failed to connect to ACP server: {e}")
    
    async def disconnect(self) -> None:
        """断开与 ACP 服务器的连接。"""
        if self._receive_task:
            self._receive_task.cancel()
            try:
                await self._receive_task
            except asyncio.CancelledError:
                pass
            self._receive_task = None
        
        if self._ws:
            try:
                await self._ws.close()
            except Exception:
                pass
            self._ws = None
        
        self._connected = False
        self._initialized = False
        logger.info("ACP disconnected")
    
    async def _receive_loop(self) -> None:
        """消息接收循环。"""
        while self._connected:
            try:
                raw = await self._ws.recv()
                
                # 跳过非 JSON 消息（如 //ready, //stderr 等）
                if isinstance(raw, str) and not raw.strip().startswith("{"):
                    logger.debug(f"ACP non-JSON message: {raw[:100]}")
                    continue
                
                message = json.loads(raw)
                
                # 处理响应或通知
                if "id" in message:
                    # 这是一个响应
                    request_id = message["id"]
                    if request_id in self._pending_requests:
                        future = self._pending_requests.pop(request_id)
                        if not future.done():
                            future.set_result(message)
                else:
                    # 这是一个通知，根据 sessionId 分发 (并行 Session 支持)
                    params = message.get("params", {})
                    session_id = params.get("sessionId")
                    if session_id and session_id in self._session_queues:
                        await self._session_queues[session_id].put(message)
                    else:
                        # 这是一个通知，放入全局队列
                        await self._message_queue.put(message)
                    
            except asyncio.CancelledError:
                break
            except websockets.ConnectionClosed:
                logger.warning("ACP WebSocket connection closed, will reconnect on next request")
                self._connected = False
                self._initialized = False
                # 不跳出循环，等待下次请求时重连
                break
            except json.JSONDecodeError as e:
                logger.debug(f"ACP JSON decode error: {e}")
                continue
            except Exception as e:
                logger.error(f"ACP receive error: {e}")
    
    def _next_request_id(self) -> int:
        """获取下一个请求 ID。"""
        self._request_id += 1
        return self._request_id
    
    async def _send_request(
        self,
        method: str,
        params: dict,
        timeout: Optional[int] = None,
    ) -> dict:
        """
        发送 JSON-RPC 请求并等待响应。
        
        Args:
            method: JSON-RPC 方法名
            params: 方法参数
            timeout: 超时时间（秒）
        
        Returns:
            响应结果
        """
        # 如果连接断开，尝试重连
        if not self._connected or not self._ws:
            logger.info("ACP connection lost, reconnecting...")
            await self.connect()
            await self.initialize()
            await self.authenticate("iflow")
        
        request_id = self._next_request_id()
        
        request = {
            "jsonrpc": "2.0",
            "id": request_id,
            "method": method,
            "params": params,
        }
        
        # 创建 Future 等待响应
        future: asyncio.Future[dict] = asyncio.get_event_loop().create_future()
        self._pending_requests[request_id] = future
        
        try:
            await self._ws.send(json.dumps(request))
            logger.debug(f"ACP request: {method} (id={request_id})")
            
            # 等待响应
            timeout = timeout or self.timeout
            response = await asyncio.wait_for(future, timeout=timeout)
            
            if "error" in response:
                error = response["error"]
                raise ACPError(f"ACP error: {error.get('message', str(error))}")
            
            return response.get("result", {})
            
        except asyncio.TimeoutError:
            self._pending_requests.pop(request_id, None)
            raise ACPTimeoutError(f"ACP request timeout: {method}")
    
    async def initialize(self) -> dict:
        """
        初始化 ACP 连接。
        
        协商协议版本并交换能力。
        """
        if self._initialized:
            return self._agent_capabilities
        
        # 客户端能力 - 使用 camelCase 格式
        client_capabilities = {
            "fs": {
                "readTextFile": True,
                "writeTextFile": True,
            }
        }
        
        # initialize 必须包含 protocolVersion 和 clientCapabilities
        result = await self._send_request("initialize", {
            "protocolVersion": self.PROTOCOL_VERSION,
            "clientCapabilities": client_capabilities,
        })
        
        self._agent_capabilities = result.get("agentCapabilities", {})
        self._initialized = True
        
        logger.info(f"ACP initialized: version={result.get('protocolVersion')}, "
                   f"capabilities={list(self._agent_capabilities.keys())}")
        
        return self._agent_capabilities
    
    async def authenticate(self, method_id: str = "iflow") -> bool:
        """
        进行认证。
        
        Args:
            method_id: 认证方法 ID (iflow, oauth-iflow, openai-compatible)
        
        Returns:
            是否认证成功
        """
        if not self._initialized:
            await self.initialize()
        
        try:
            result = await self._send_request("authenticate", {
                "methodId": method_id,
            })
            success = result.get("methodId") == method_id
            if success:
                logger.info(f"ACP authenticated with method: {method_id}")
            return success
        except ACPError as e:
            logger.error(f"ACP authentication failed: {e}")
            return False
    
    async def create_session(
        self,
        workspace: Optional[Path] = None,
        model: Optional[str] = None,
        system_prompt: Optional[str] = None,
        approval_mode: str = "yolo",
    ) -> str:
        """
        创建新会话。
        
        Args:
            workspace: 工作目录
            model: 模型名称
            system_prompt: 系统提示
            approval_mode: 审批模式 (default, smart, yolo, plan)
        
        Returns:
            会话 ID
        """
        if not self._initialized:
            await self.initialize()
        
        ws_path = str(workspace or self.workspace or Path.cwd())
        
        # 按照正确的 ACP 协议格式
        # session/new 需要: cwd, mcpServers (必须)
        params: dict = {
            "cwd": ws_path,
            "mcpServers": [],  # 必须参数，空数组表示不使用 MCP
        }
        
        # settings 中包含 approval_mode 等
        settings: dict = {}
        if approval_mode:
            settings["permission_mode"] = approval_mode
        if model:
            settings["model"] = model
        if system_prompt:
            settings["system_prompt"] = system_prompt
        
        if settings:
            params["settings"] = settings
        
        result = await self._send_request("session/new", params)
        session_id = result.get("sessionId", "")
        
        if model:
            try:
                await self._send_request("session/set_model", {
                    "sessionId": session_id,
                    "modelId": model,
                }, timeout=10)
                logger.debug(f"Set model to {model} for session via set_model")
            except Exception as e:
                logger.warning(f"Failed to set model via session/set_model: {e}, trying set_config_option")
                try:
                    await self._send_request("session/set_config_option", {
                        "sessionId": session_id,
                        "configId": "model",
                        "value": model,
                    }, timeout=10)
                    logger.debug(f"Set model to {model} via set_config_option")
                except Exception as e2:
                    logger.debug(f"Failed to set model via set_config_option: {e2}")
        
        logger.info(f"ACP session created: {session_id[:16] if session_id else 'unknown'}...")
        
        return session_id
    
    async def load_session(self, session_id: str) -> bool:
        """
        加载已有会话。
        
        Args:
            session_id: 会话 ID
        
        Returns:
            是否成功加载
        """
        if not self._initialized:
            await self.initialize()
        
        try:
            # session/load 需要 cwd 和 mcpServers 参数
            result = await self._send_request("session/load", {
                "sessionId": session_id,
                "cwd": str(self.workspace),
                "mcpServers": [],
            })
            return result.get("loaded", False)
        except ACPError as e:
            logger.warning(f"Failed to load session {session_id[:16]}...: {e}")
            return False
    
    async def prompt(
        self,
        session_id: str,
        message: str,
        timeout: Optional[int] = None,
        on_chunk: Optional[Callable[[AgentMessageChunk], None]] = None,
        on_tool_call: Optional[Callable[[ToolCall], None]] = None,
        on_event: Optional[Callable[[dict[str, Any]], Coroutine]] = None,
    ) -> ACPResponse:
        """
        发送消息并获取响应。
        
        Args:
            session_id: 会话 ID
            message: 用户消息
            timeout: 超时时间（秒）
            on_chunk: 消息块回调
            on_tool_call: 工具调用回调
        
        Returns:
            ACP 响应结果
        """
        if not self._connected:
            raise ACPConnectionError("Not connected to ACP server")
        
        response = ACPResponse()
        content_parts: list[str] = []
        thought_parts: list[str] = []
        tool_calls_map: dict[str, ToolCall] = {}
        
        # 发送 prompt 请求 - 使用正确的参数格式
        # session/prompt 需要: sessionId, prompt (数组)
        request_id = self._next_request_id()
        
        request = {
            "jsonrpc": "2.0",
            "id": request_id,
            "method": "session/prompt",
            "params": {
                "sessionId": session_id,
                "prompt": [
                    {"type": "text", "text": message}
                ],
            },
        }
        
        future: asyncio.Future[dict] = asyncio.get_event_loop().create_future()
        self._pending_requests[request_id] = future
        
        # 为当前 session 注册专用消息队列
        session_queue = asyncio.Queue()
        self._session_queues[session_id] = session_queue
        
        async with self._prompt_lock:
            try:
                await self._ws.send(json.dumps(request))
                logger.debug(f"ACP prompt sent (session={session_id[:16]}...)")
            except Exception as e:
                self._pending_requests.pop(request_id, None)
                self._session_queues.pop(session_id, None)
                raise e
        
        try:
            # 接收更新直到收到最终响应
            timeout = timeout or self.timeout
            start_time = time.time()
            
            while True:
                remaining = timeout - (time.time() - start_time)
                if remaining <= 0:
                    raise ACPTimeoutError("Prompt timeout")
                
                try:
                    # 优先检查最终响应
                    if future.done():
                        break
                    
                    # 优先检查自己的私有队列，找不到再看全局队列
                    try:
                        msg = await asyncio.wait_for(
                            session_queue.get(),
                            timeout=0.1
                        )
                    except asyncio.TimeoutError:
                        # 等待全局通知消息
                        msg = await asyncio.wait_for(
                            self._message_queue.get(),
                            timeout=min(remaining, 4.9)
                        )
                    
                    # 处理 session/update 通知
                    if msg.get("method") == "session/update":
                        params = msg.get("params", {})
                        update = params.get("update", {})
                        update_type = update.get("sessionUpdate", "")
                        if on_event:
                            await on_event(
                                {
                                    "session_id": session_id,
                                    "update_type": update_type,
                                    "update": update,
                                }
                            )
                        
                        if update_type == "agent_message_chunk":
                            # Agent 消息块 - content 是一个 dict，不是数组
                            content = update.get("content", {})
                            if isinstance(content, dict) and content.get("type") == "text":
                                chunk_text = content.get("text", "")
                                if chunk_text:
                                    content_parts.append(chunk_text)
                                    if on_chunk:
                                        await on_chunk(AgentMessageChunk(text=chunk_text))
                        
                        elif update_type == "agent_thought_chunk":
                            # Agent 思考过程
                            content = update.get("content", {})
                            if isinstance(content, dict) and content.get("type") == "text":
                                chunk_text = content.get("text", "")
                                if chunk_text:
                                    thought_parts.append(chunk_text)
                                    if on_chunk:
                                        await on_chunk(AgentMessageChunk(text=chunk_text, is_thought=True))
                        
                        elif update_type == "tool_call":
                            # 新的工具调用
                            tool_call_id = update.get("toolCallId", "")
                            tool_name = update.get("name", "")
                            args = update.get("args", {})
                            
                            tc = ToolCall(
                                tool_call_id=tool_call_id,
                                tool_name=tool_name,
                                status="pending",
                                args=args,
                            )
                            tool_calls_map[tool_call_id] = tc
                            
                            if on_tool_call:
                                await on_tool_call(tc)
                        
                        elif update_type == "tool_call_update":
                            # 工具调用更新
                            tool_call_id = update.get("toolCallId", "")
                            status = update.get("status", "")
                            output_text = ""
                            
                            content = update.get("content", [])
                            if isinstance(content, list):
                                for c in content:
                                    if c.get("type") == "text":
                                        output_text += c.get("text", "")
                            elif isinstance(content, dict) and content.get("type") == "text":
                                output_text = content.get("text", "")
                            
                            if tool_call_id in tool_calls_map:
                                tc = tool_calls_map[tool_call_id]
                                if status:
                                    tc.status = status
                                if output_text:
                                    tc.output = output_text
                                
                                if on_tool_call:
                                    await on_tool_call(tc)
                
                except asyncio.TimeoutError:
                    # 继续检查最终响应
                    continue
                
                # 检查最终响应
                if future.done():
                    break
            
            # 彻底收割最后可能残留的更新消息
            while True:
                try:
                    try:
                        msg = await asyncio.wait_for(session_queue.get(), timeout=0.1)
                    except asyncio.TimeoutError:
                        msg = await asyncio.wait_for(self._message_queue.get(), timeout=0.1)
                        
                    if msg.get("method") == "session/update":
                        params = msg.get("params", {})
                        update = params.get("update", {})
                        update_type = update.get("sessionUpdate", "")
                        if on_event:
                            await on_event(
                                {
                                    "session_id": session_id,
                                    "update_type": update_type,
                                    "update": update,
                                }
                            )
                        
                        if update_type == "agent_message_chunk":
                            content = update.get("content", {})
                            if isinstance(content, dict) and content.get("type") == "text":
                                chunk_text = content.get("text", "")
                                if chunk_text:
                                    content_parts.append(chunk_text)
                                    if on_chunk:
                                        await on_chunk(AgentMessageChunk(text=chunk_text))
                except asyncio.TimeoutError:
                    break
                    
            # 获取最终响应
            final_response = future.result()
                
            if "error" in final_response:
                response.error = final_response["error"].get("message", str(final_response["error"]))
                response.stop_reason = StopReason.ERROR
            else:
                result = final_response.get("result", {})
                stop_reason_str = result.get("stopReason", "end_turn")
                try:
                    response.stop_reason = StopReason(stop_reason_str)
                except ValueError:
                    response.stop_reason = StopReason.END_TURN
            
            # 组装响应
            response.content = "".join(content_parts)
            response.thought = "".join(thought_parts)
            response.tool_calls = list(tool_calls_map.values())
            
            logger.debug(f"ACP prompt completed: stop_reason={response.stop_reason}, "
                        f"content_len={len(response.content)}")
            
            return response
            
        except asyncio.TimeoutError:
            self._pending_requests.pop(request_id, None)
            raise ACPTimeoutError("Prompt timeout")
        except Exception as e:
            self._pending_requests.pop(request_id, None)
            raise ACPError(f"Prompt error: {e}")
        finally:
            # 清理
            self._session_queues.pop(session_id, None)
            self._pending_requests.pop(request_id, None)
    
    async def cancel(self, session_id: str) -> None:
        """
        取消当前请求。
        
        Args:
            session_id: 会话 ID
        """
        if not self._connected or not self._ws:
            return
        
        notification = {
            "jsonrpc": "2.0",
            "method": "session/cancel",
            "params": {
                "sessionId": session_id,
            },
        }
        
        try:
            await self._ws.send(json.dumps(notification))
            logger.debug(f"ACP cancel sent (session={session_id[:16]}...)")
        except Exception as e:
            logger.warning(f"Failed to send cancel: {e}")
    
    async def is_connected(self) -> bool:
        """检查连接状态。"""
        return self._connected and self._ws is not None


class ACPAdapter:
    """
    ACP 适配器 - 封装 ACPClient 提供与 IFlowAdapter 兼容的接口。
    
    管理会话映射，提供 chat 和 new_chat 方法。
    """
    
    def __init__(
        self,
        host: str = "localhost",
        port: int = 8090,
        timeout: int = DEFAULT_TIMEOUT,
        workspace: Optional[Path] = None,
        default_model: str = "glm-5",
        thinking: bool = False,
    ):
        """
        初始化 ACP 适配器。
        
        Args:
            host: ACP 服务器主机地址
            port: ACP 服务器端口
            timeout: 请求超时时间（秒）
            workspace: 工作目录路径
            default_model: 默认模型
            thinking: 是否启用思考模式
        """
        self.host = host
        self.port = port
        self.timeout = timeout
        self.workspace = workspace
        self.default_model = default_model
        self.thinking = thinking
        
        self._client: Optional[ACPClient] = None
        self._session_map: dict[str, str] = {}  # channel:chat_id -> session_id
        self._session_map_file = Path.home() / ".iflow-bot" / "session_mappings.json"
        self._session_lock = asyncio.Lock()  # 防止并发创建 session
        self._load_session_map()
        
        logger.info(f"ACPAdapter: host={host}, port={port}, workspace={workspace}")
    
    def _load_session_map(self) -> None:
        """加载持久化的 session 映射。"""
        if self._session_map_file.exists():
            try:
                with open(self._session_map_file, "r", encoding="utf-8") as f:
                    self._session_map = json.load(f)
                logger.debug(f"Loaded {len(self._session_map)} session mappings")
            except json.JSONDecodeError:
                logger.warning("Invalid session mapping file, starting fresh")
                self._session_map = {}
    
    def _save_session_map(self) -> None:
        """保存 session 映射到文件。"""
        self._session_map_file.parent.mkdir(parents=True, exist_ok=True)
        with open(self._session_map_file, "w", encoding="utf-8") as f:
            json.dump(self._session_map, f, indent=2, ensure_ascii=False)
    
    def _find_session_file(self, session_id: str) -> Optional[Path]:
        """查找 session 对话历史文件。
        
        Args:
            session_id: session ID
        
        Returns:
            session 文件路径，如果不存在返回 None
        """
        # ACP sessions 存放在 ~/.iflow/acp/sessions/
        sessions_dir = Path.home() / ".iflow" / "acp" / "sessions"
        
        if not sessions_dir.exists():
            return None
        
        # 文件名格式: {session_id}.json
        session_file = sessions_dir / f"{session_id}.json"
        if session_file.exists():
            return session_file
        
        return None
    
    def _extract_conversation_history(
        self,
        session_id: str,
        max_turns: int = 20,
        token_budget: int = 3000,
    ) -> Optional[str]:
        """提取 session 对话历史并格式化成提示词。
        
        Args:
            session_id: session ID
            max_turns: 最大提取的对话轮次
        
        Returns:
            格式化后的对话历史，如果无法提取返回 None
        """
        import datetime
        
        session_file = self._find_session_file(session_id)
        if not session_file:
            logger.debug(f"Session file not found for: {session_id[:16]}...")
            return None
        
        try:
            with open(session_file, "r", encoding="utf-8") as f:
                data = json.load(f)
            
            chat_history = data.get("chatHistory", [])
            if not chat_history:
                return None
            
            all_conversations: list[tuple[str, str, str]] = []
            for chat in chat_history:
                role = chat.get("role")
                parts = chat.get("parts", [])
                
                # 提取文本内容
                full_text = ""
                for part in parts:
                    if isinstance(part, dict):
                        text = part.get("text", "")
                        if text:
                            full_text += text + "\n"
                
                if not full_text.strip():
                    continue
                
                # 对于用户消息，提取"用户消息:"之后的实际内容
                if role == "user":
                    # 尝试提取用户消息后的实际内容
                    if "用户消息:" in full_text:
                        # 取用户消息:之后的内容
                        idx = full_text.find("用户消息:") + len("用户消息:")
                        content = full_text[idx:].strip()
                    else:
                        # 如果没有"用户消息:"，跳过系统提示
                        continue
                    
                    # 跳过太短或太长的内容
                    if len(content) < 2 or len(content) > 2000:
                        continue
                    
                    # 获取时间戳
                    timestamp = chat.get("timestamp") or data.get("createdAt", "")
                    time_str = ""
                    if timestamp:
                        try:
                            ts = timestamp.replace("Z", "+00:00")
                            dt = datetime.datetime.fromisoformat(ts.replace("+00:00", ""))
                            time_str = dt.strftime("%Y-%m-%d %H:%M:%S")
                        except:
                            pass
                    all_conversations.append(("user", time_str, content))
                
                elif role == "model":
                    # 对于 model 响应，过滤掉系统提示
                    content = full_text.strip()
                    
                    # 过滤掉太长的内容
                    if len(content) > 3000:
                        content = content[:3000] + "..."
                    
                    # 跳过包含系统提示的内容
                    if "<system-reminder>" in content or "[AGENTS - 工作空间指南]" in content:
                        continue
                    
                    if len(content) > 10:
                        all_conversations.append(("assistant", "", content))
            
            if not all_conversations:
                return None
            
            history = self._build_budgeted_history_context(
                all_conversations,
                token_budget=token_budget,
                recent_turns=max_turns,
            )
            logger.info(f"Extracted {len(all_conversations)} conversation turns from session {session_id[:16]}...")
            return history
            
        except Exception as e:
            logger.warning(f"Failed to extract conversation history: {e}")
            return None
    
    async def connect(self) -> None:
        """连接到 ACP 服务器并进行认证。"""
        if self._client is None:
            self._client = ACPClient(
                host=self.host,
                port=self.port,
                timeout=self.timeout,
                workspace=self.workspace,
            )
        
        await self._client.connect()
        await self._client.initialize()
        
        # 进行认证 (使用 iflow 方法，需要设置 IFLOW_API_KEY 环境变量)
        authenticated = await self._client.authenticate("iflow")
        if not authenticated:
            logger.warning("ACP authentication failed, some features may not work")
    
    async def disconnect(self) -> None:
        """断开连接。"""
        if self._client:
            await self._client.disconnect()
            self._client = None
    
    def _get_session_key(self, channel: str, chat_id: str) -> str:
        """获取会话键。"""
        return f"{channel}:{chat_id}"

    @staticmethod
    def _inject_history_before_user_message(message: str, history_context: str) -> str:
        if not history_context:
            return message
        user_msg_marker = "用户消息:"
        if user_msg_marker in message:
            idx = message.find(user_msg_marker)
            return message[:idx] + history_context + "\n\n" + message[idx:]
        return f"{history_context}\n\n{message}"

    @staticmethod
    def _estimate_tokens(text: str) -> int:
        if not text:
            return 0
        return max(1, len(text) // 4)

    @staticmethod
    def _clip_text(text: str, max_chars: int) -> str:
        clean = (text or "").strip()
        if len(clean) <= max_chars:
            return clean
        return clean[:max_chars].rstrip() + "..."

    def _build_budgeted_history_context(
        self,
        conversations: list[tuple[str, str, str]],
        token_budget: int = 3000,
        recent_turns: int = 20,
    ) -> str:
        if not conversations:
            return "<history_context>\n</history_context>"

        def fmt(role: str, time_str: str, content: str, max_chars: int) -> str:
            body = self._clip_text(content, max_chars)
            if role == "user":
                return f"{time_str}\n用户：{body}".strip()
            return f"我：{body}"

        recent = conversations[-recent_turns:] if recent_turns > 0 else []
        older = conversations[:-len(recent)] if recent else conversations[:]

        summary_lines: list[str] = []
        if older:
            summary_lines.append("更早对话摘要：")
            for role, time_str, content in older[-40:]:
                prefix = "用户" if role == "user" else "我"
                item = self._clip_text(content, 120)
                if time_str:
                    summary_lines.append(f"- {time_str} {prefix}: {item}")
                else:
                    summary_lines.append(f"- {prefix}: {item}")

        blocks = [fmt(role, time_str, content, 1600) for role, time_str, content in recent]
        summary_block = "\n".join(summary_lines).strip()

        def build_text(cur_summary: str, cur_blocks: list[str]) -> str:
            parts: list[str] = []
            if cur_summary:
                parts.append(cur_summary)
            parts.extend([b for b in cur_blocks if b.strip()])
            return "<history_context>\n" + "\n\n".join(parts).strip() + "\n</history_context>"

        text = build_text(summary_block, blocks)

        while self._estimate_tokens(text) > token_budget and len(blocks) > 4:
            blocks.pop(0)
            text = build_text(summary_block, blocks)

        if self._estimate_tokens(text) > token_budget and summary_block:
            compact_summary_lines = []
            for line in summary_lines[:24]:
                compact_summary_lines.append(self._clip_text(line, 90))
            summary_block = "\n".join(compact_summary_lines)
            text = build_text(summary_block, blocks)

        while self._estimate_tokens(text) > token_budget and len(text) > 200:
            text = text[: max(200, int(len(text) * 0.9))].rstrip() + "\n</history_context>"

        return text

    @staticmethod
    def _is_context_overflow_error(error_text: str) -> bool:
        text = (error_text or "").lower()
        keywords = [
            "context",
            "token",
            "too long",
            "max_tokens",
            "max token",
            "exceed",
            "length",
        ]
        return any(keyword in text for keyword in keywords)
    
    async def _get_or_create_session(
        self,
        channel: str,
        chat_id: str,
        model: Optional[str] = None,
    ) -> str:
        """
        获取或创建会话。
        
        Args:
            channel: 渠道名称
            chat_id: 聊天 ID
            model: 模型名称
        
        Returns:
            会话 ID
        """
        key = self._get_session_key(channel, chat_id)
        
        # 如果 session 已存在，直接返回（不尝试 load，因为 ACP session 在服务端保持）
        if key in self._session_map:
            logger.debug(f"Reusing existing session: {key} -> {self._session_map[key][:16]}...")
            return self._session_map[key]
        
        return await self._create_new_session(key, model)
    
    async def _create_new_session(
        self,
        key: str,
        model: Optional[str] = None,
    ) -> str:
        """创建新会话。"""
        async with self._session_lock:
            # 再次检查，可能在等待锁时其他协程已创建
            if key in self._session_map:
                logger.debug(f"Session created by another request: {key} -> {self._session_map[key][:16]}...")
                return self._session_map[key]
            
            # 创建新会话
            if not self._client:
                raise ACPConnectionError("ACP client not connected")
            
            session_id = await self._client.create_session(
                workspace=self.workspace,
                model=model or self.default_model,
                approval_mode="yolo",
            )
            
            self._session_map[key] = session_id
            self._save_session_map()
            logger.info(f"ACP session mapped: {key} -> {session_id[:16]}...")
            
            return session_id
    
    async def _invalidate_session(self, key: str) -> Optional[str]:
        """使 session 失效并清除映射。
        
        Returns:
            旧的 session_id，如果不存在返回 None
        """
        old_session = self._session_map.pop(key, None)
        if old_session:
            self._save_session_map()
            logger.info(f"Session invalidated: {key} -> {old_session[:16]}...")
        return old_session
    
    async def _get_or_create_session_with_retry(
        self,
        channel: str,
        chat_id: str,
        model: Optional[str] = None,
    ) -> str:
        """获取或创建 session，如果失效则自动重建。"""
        key = self._get_session_key(channel, chat_id)
        
        if key in self._session_map:
            return self._session_map[key]
        
        return await self._create_new_session(key, model)
    
    async def chat(
        self,
        message: str,
        channel: str = "cli",
        chat_id: str = "direct",
        model: Optional[str] = None,
        timeout: Optional[int] = None,
    ) -> str:
        """
        发送消息并获取响应。
        
        Args:
            message: 用户消息
            channel: 渠道名称
            chat_id: 聊天 ID
            model: 模型名称
            timeout: 超时时间
        
        Returns:
            响应文本
        """
        if not self._client:
            raise ACPConnectionError("ACP client not connected")
        
        key = self._get_session_key(channel, chat_id)
        session_id = await self._get_or_create_session(channel, chat_id, model)
        
        response = await self._client.prompt(
            session_id=session_id,
            message=message,
            timeout=timeout or self.timeout,
        )
        
        # 如果 session 失效，自动重建并重试
        if response.error and "Invalid request" in response.error:
            logger.warning(f"Session invalid, recreating: {key}")
            # 获取旧的 session_id 并提取对话历史
            old_session_id = await self._invalidate_session(key)
            history_context = ""
            if old_session_id:
                history_context = self._extract_conversation_history(old_session_id) or ""
            
            # 创建新 session 并注入历史
            session_id = await self._create_new_session(key, model)
            
            # 如果有历史，注入到消息中（放在"用户消息:"之前）
            if history_context:
                message = self._inject_history_before_user_message(message, history_context)
                logger.info(f"Injected conversation history before user message")
            
            response = await self._client.prompt(
                session_id=session_id,
                message=message,
                timeout=timeout or self.timeout,
            )

        if response.error and self._is_context_overflow_error(response.error):
            logger.warning(f"Context overflow suspected, recreating with compact history: {key}")
            old_session_id = await self._invalidate_session(key)
            history_context = ""
            if old_session_id:
                history_context = self._extract_conversation_history(
                    old_session_id,
                    max_turns=4,
                    token_budget=1200,
                ) or ""

            session_id = await self._create_new_session(key, model)
            retry_message = message
            if history_context:
                retry_message = self._inject_history_before_user_message(retry_message, history_context)
                logger.info("Injected compact conversation history before user message")

            response = await self._client.prompt(
                session_id=session_id,
                message=retry_message,
                timeout=timeout or self.timeout,
            )
        
        if response.error:
            raise ACPError(f"Chat error: {response.error}")
        
        # 如果启用思考模式，包含思考过程
        if self.thinking and response.thought:
            return f"[Thinking]\n{response.thought}\n\n[Response]\n{response.content}"
        
        return response.content
    
    async def chat_stream(
        self,
        message: str,
        channel: str = "cli",
        chat_id: str = "direct",
        model: Optional[str] = None,
        timeout: Optional[int] = None,
        on_chunk: Optional[Callable[[AgentMessageChunk], Coroutine]] = None,
        on_tool_call: Optional[Callable[[ToolCall], Coroutine]] = None,
        on_event: Optional[Callable[[dict[str, Any]], Coroutine]] = None,
    ) -> str:
        """
        发送消息并流式获取响应。
        
        Args:
            message: 用户消息
            channel: 渠道名称
            chat_id: 聊天 ID
            model: 模型名称
            timeout: 超时时间
            on_chunk: 消息块回调（支持异步）
            on_tool_call: 工具调用回调（支持异步）
        
        Returns:
            最终响应文本
        """
        if not self._client:
            raise ACPConnectionError("ACP client not connected")
        
        key = self._get_session_key(channel, chat_id)
        session_id = await self._get_or_create_session(channel, chat_id, model)
        
        # 收集完整响应
        content_parts: list[str] = []
        
        async def handle_chunk(chunk: AgentMessageChunk):
            """处理消息块。"""
            if not chunk.is_thought and chunk.text:
                content_parts.append(chunk.text)
            if on_chunk:
                result = on_chunk(chunk)
                if asyncio.iscoroutine(result):
                    await result
        
        async def handle_tool_call(tool_call: ToolCall):
            """处理工具调用。"""
            if on_tool_call:
                result = on_tool_call(tool_call)
                if asyncio.iscoroutine(result):
                    await result

        async def handle_event(event: dict[str, Any]):
            if on_event:
                result = on_event(event)
                if asyncio.iscoroutine(result):
                    await result

        response = await self._client.prompt(
            session_id=session_id,
            message=message,
            timeout=timeout or self.timeout,
            on_chunk=handle_chunk,
            on_tool_call=handle_tool_call,
            on_event=handle_event,
        )
        
        # 如果 session 失效，自动重建并重试
        if response.error and "Invalid request" in response.error:
            logger.warning(f"Session invalid (stream), recreating: {key}")
            # 获取旧的 session_id 并提取对话历史
            old_session_id = await self._invalidate_session(key)
            history_context = ""
            if old_session_id:
                history_context = self._extract_conversation_history(old_session_id) or ""
            
            # 创建新 session 并注入历史
            session_id = await self._create_new_session(key, model)
            
            # 如果有历史，注入到消息中（放在"用户消息:"之前）
            if history_context:
                message = self._inject_history_before_user_message(message, history_context)
                logger.info(f"Injected conversation history before user message (stream)")
            
            content_parts.clear()
            response = await self._client.prompt(
                session_id=session_id,
                message=message,
                timeout=timeout or self.timeout,
                on_chunk=handle_chunk,
                on_tool_call=handle_tool_call,
                on_event=handle_event,
            )
        
        stream_content = "".join(content_parts) or response.content

        should_recover = False
        if response.error and self._is_context_overflow_error(response.error):
            should_recover = True
            logger.warning(f"Context overflow suspected (stream), recreating with compact history: {key}")
        elif not (stream_content or "").strip():
            should_recover = True
            logger.warning(f"Empty stream response detected, recreating with compact history: {key}")

        if should_recover:
            old_session_id = await self._invalidate_session(key)
            history_context = ""
            if old_session_id:
                history_context = self._extract_conversation_history(
                    old_session_id,
                    max_turns=4,
                    token_budget=1200,
                ) or ""

            session_id = await self._create_new_session(key, model)
            retry_message = message
            if history_context:
                retry_message = self._inject_history_before_user_message(retry_message, history_context)
                logger.info("Injected compact conversation history before user message (stream)")

            content_parts.clear()
            response = await self._client.prompt(
                session_id=session_id,
                message=retry_message,
                timeout=timeout or self.timeout,
                on_chunk=handle_chunk,
                on_tool_call=handle_tool_call,
                on_event=handle_event,
            )
            stream_content = "".join(content_parts) or response.content

        if response.error:
            raise ACPError(f"Chat error: {response.error}")

        # 返回收集的完整响应
        return stream_content
    
    async def new_chat(
        self,
        message: str,
        channel: str = "cli",
        chat_id: str = "direct",
        model: Optional[str] = None,
        timeout: Optional[int] = None,
    ) -> str:
        """
        开始新对话。
        
        清除已有会话映射，创建新会话。
        """
        key = self._get_session_key(channel, chat_id)
        if key in self._session_map:
            del self._session_map[key]
        
        return await self.chat(message, channel, chat_id, model, timeout)
    
    async def health_check(self) -> bool:
        """检查连接状态。"""
        if self._client is None:
            return False
        return await self._client.is_connected()
    
    def clear_session(self, channel: str, chat_id: str) -> bool:
        """清除会话映射。"""
        key = self._get_session_key(channel, chat_id)
        if key in self._session_map:
            del self._session_map[key]
            return True
        return False
    
    def list_sessions(self) -> dict[str, str]:
        """列出所有会话映射。"""
        return self._session_map.copy()
